@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------*/
 /* Program to get/set some ACPI settings                                     */
 /* Author : David Leemans   -   http://freeunix.dyndns.org:8000/             */
-/* Last update : 31-01-2006                                                  */
+/* Last update : 14-05-2006                                                  */
 /*---------------------------------------------------------------------------*/
 /*                                                                           */
 /* acpitool.cpp                                                              */
@@ -37,8 +37,8 @@
 
 using namespace std;
 
-#define App_Version  "AcpiTool v0.4.4"
-#define Release_Date "05-Feb-2006"
+#define App_Version  "AcpiTool v0.4.5"
+#define Release_Date "14-May-2006"
 
 int Kernel_24 = 0, Kernel_26 = 0;
 
@@ -264,7 +264,7 @@ int Do_Battery_Stuff(int show_empty, int info_level, int verbose)
 		Total_Rate = Total_Rate + (atoi(Batt_Info[t]->Present_Rate));
 	    }  
 	    
-	    if(Total_Rate > 0)           // avoid division by 0 if all batteries are charged //
+	    if(Total_Rate > 0)    // unlikely to happen if we got here but avoid possible division by 0 //
 	    {
 		Remaining_Percentage = float(Total_Remaining_Cap) / float(Total_LastFull_Cap) * 100.0;
 		
@@ -692,7 +692,9 @@ int Show_CPU_Info()
 	DIR *cpu_dir;
 	char *name;
 
+	//file_in.open("/home/david/dropzone/devel/acpi/processor/CPU0/cpuinfo");//
 	file_in.open("/proc/cpuinfo");
+
 	if(file_in)                   
 	{
 		for(int t=0; t<4; t++) file_in.getline(str, 80);          //skip 4 lines //
@@ -700,12 +702,23 @@ int Show_CPU_Info()
 		bzero(temp, 80);
 		strncpy(temp, str+13, 49);
 		cout<<"  CPU type               : "<<temp<<endl;
+		
+		int skip = 10;
+		if(strncmp(temp,"AMD Athlon(tm) 64",17)==0) skip = 6;    // cpuinfo returns diiferent info for AMD64 cpu's //	
+		
 		file_in.getline(str, 100);
 		file_in.getline(str, 100);
 		bzero(temp, 80);
 		strncpy(temp, str+11, 10);
 		cout<<"  CPU speed              : "<<temp<<" MHz"<<endl;
-		for(int t=0; t<11; t++) file_in.getline(str,199);        //skip 10 lines //
+		
+		file_in.getline(str, 100);
+		bzero(temp, 80);
+		strncpy(temp, str+13, 10);
+		cout<<"  Cache size             : "<<temp<<endl;
+		
+		for(int t=0; t<skip; t++) file_in.getline(str,199);        // skip "skip" lines to reach Bogomips line//
+		
 		bzero(temp, 80);
 		strncpy(temp, str+11, 10);
 		cout<<"  Bogomips               : "<<temp<<endl;
@@ -766,7 +779,10 @@ int Show_CPU_Info()
 
 		for(int i=findex; i>0; i-=3)     //process entries in reverse order //
 		{
-			file_in.open(filename[i-2]); // acpi/processor/%s/info
+			file_in.open(filename[i-2]);      // acpi/processor/%s/info
+			
+			//file_in.open("/home/david/dropzone/devel/acpi/processor/CPU0/info");//
+			
 			if(file_in)                   
 			{
 				file_in.getline(str, 90); // processor id
@@ -806,6 +822,8 @@ int Show_CPU_Info()
 			}	
 
 			file2_in.open(filename[i-1]); // acpi/processor/%s/power
+			//file2_in.open("/home/david/dropzone/devel/acpi/processor/CPU0/power");//
+			
 			if(file2_in)                   
 			{
 				file2_in.getline(str, 100); // active state
@@ -823,6 +841,7 @@ int Show_CPU_Info()
 				// maybe this will be fixed at some later time...
 				// cout << "  Supported C-states     : " << max_cstate << endl;
 
+
 				file2_in.getline(str, 100); // bus master activity
 				memset(&temp, 0, sizeof(temp));
 				strncpy(temp, str+25, 8);
@@ -833,7 +852,8 @@ int Show_CPU_Info()
 				file2_in.getline(str, 100); // states (line contains no information)
 				while(!file2_in.eof())
 				{
-					file2_in.getline(str, 100);  // cstate Cx
+					file2_in.getline(str, 130);  // cstate Cx
+								     // reading up to 130 char here for AMD64 cpu's  //
 					if (strlen(str)!=0)
 					{
 						cst_cnt++; // NOTE: we have no usage cnter for C0!!!
@@ -848,11 +868,14 @@ int Show_CPU_Info()
 				cout << "  C-states (incl. C0)    : " << (max_cstate + 1) << endl;
 
 				// calculate usage etc.
-				for (int cst=1; cst <= max_cstate ; cst++)
+				if(cst_usage_notC0!=0)  // avoid possible division by 0, happens if usage is 00000000 //
 				{
+				    for (int cst=1; cst <= max_cstate ; cst++)
+				    {
 					cst_usage_rel[cst] = float(cst_usage_abs[cst]) / cst_usage_notC0;
 					fprintf(stdout, "  Usage of state C%d      : %li (%.1f %%)\n", cst,
 							cst_usage_abs[cst], (cst_usage_rel[cst] * 100));
+				    }
 				}
 
 				file2_in.close();
